@@ -7,7 +7,7 @@ use diagnostics;
 require Exporter;
 our @ISA = qw(Exporter);
 our @EXPORT = qw( auction );
-our $VERSION = '0.80';
+our $VERSION = '0.81';
 
 #Variables global to the package	
 my $maximize_total_benefit;
@@ -99,13 +99,13 @@ sub auction { #						=> default values
 	$inicial_price    = $args{inicial_price};
 	$price_object{$_} = $inicial_price for ( 0 .. $max_size - 1 );
 	#$price_object{$_} = sprintf( "%.0f", rand($max_matrix_value) ) for ( 0 .. $max_size - 1 ); # random values for initial prices	
-   
+	
 	# inicial epsilon value
 	my $epsilon = int (($max_matrix_value/2) * exp ( -1 * $max_size/$min_size ));  # exp (1) = e = 2.71828182845905
 	   $epsilon = $args{inicial_stepsize} if ( defined $args{inicial_stepsize} );
 	   $epsilon = 1/(1+$min_size) if ($epsilon < 1/$min_size);
     
-	my ( @assignment, @prices, %same_benefit );
+	my ( @assignment, @prices );
 	my $feasible_assignment_condition = 0;
 
 	# The preceding observations suggest the idea of epsilon-scaling, which consists of applying the algorithm several times, 
@@ -432,7 +432,7 @@ sub get_matrix_info {
 sub auctionRound {
 	my ( $matrix_ref, $epsilon, $verbose ) = @_;
 	my @matrix = @$matrix_ref;
-	my ( %info, %seen_bid );
+	my %info;
 
 	if ( $verbose >= 8 ){
 		print $output "\n Start: Matrix Size N x M: $min_size x $max_size ; Num Iterations: $iter_count_global ; epsilon_scaling = $epsilon_scaling ; iter_count_local = $iter_count_local ; epsilon: $epsilon \n";
@@ -467,7 +467,7 @@ sub auctionRound {
 			my ( $Opt01ObjForPersonI_new_list, $Opt02ObjForPersonI_new_list, $Opt03ObjForPersonI_new_list );
 			my ( $Opt01ValForPersonI_new_list, $Opt02ValForPersonI_new_list, $Opt03ValForPersonI_new_list ) = ( -10 * exp ($max_matrix_value), -10 * exp ($max_matrix_value), -10 * exp ($max_matrix_value) );
 
-			my ( $bid01ForPersonI, $bid02ForPersonI );
+			my $bid01ForPersonI;
 			my %current_value;
 			my @updated_price;
 			
@@ -520,7 +520,7 @@ sub auctionRound {
 			
 			if ( not @updated_price and not $locked_list{$person} ) # if all prices are outdated
 			{				
-				if ( $epsilon_scaling <= 2 or $epsilon > 64 ) # 1000/(4**2) ~ 64 ; 1000000/(4**7) ~ 64 ; $max_matrix_value/(4**x) ~ 4**3 --> $max_matrix_value/(4**3) ~ (4**x) --> x ~ [ln ($max_matrix_value/(4**3))]/[ln(4)]
+				if ( $epsilon_scaling <= 2 or $epsilon > 256 )
 				{
 					for my $object ( 0 .. $max_size - 1 ) # generate new list
 					{
@@ -560,7 +560,7 @@ sub auctionRound {
 						}				
 					}
 				} 
-				elsif ( ($epsilon_scaling >= 3 and $epsilon_scaling <= 6) or $epsilon > 1 )
+				elsif ( ($epsilon_scaling >= 3 and $epsilon_scaling <= 5) or $epsilon > 16 ) # in most cases, only one object is found in the complementary subset <new list> that has high value.
 				{
 					for my $object ( 0 .. $max_size - 1 ) # generate new list
 					{
@@ -591,7 +591,7 @@ sub auctionRound {
 						}				
 					}
 				}
-				elsif ( $epsilon_scaling >= 7 ) # in most cases, only one object is found in the complementary subset <new list> that has high value.
+				elsif ( $epsilon_scaling >= 6 ) # in most cases, only one object is found in the complementary subset <new list> that has high value.
 				{
 					for my $object ( 0 .. $max_size - 1 ) # generate new list
 					{
@@ -662,14 +662,14 @@ sub auctionRound {
 			
 			$bid01ForPersonI = $Opt01ValForPersonI - $Opt02ValForPersonI + $epsilon;
 			
-			# Stores the bidding info for future use
-			$info{$Opt01ObjForPersonI}{$bid01ForPersonI} = $person if ( not defined $info{$Opt01ObjForPersonI}{$bid01ForPersonI} ); # get only one person with this bid			
-	
-			$bid02ForPersonI = $Opt02ValForPersonI - $Opt03ValForPersonI + $epsilon; # calculate $bid02ForPersonI to try to predict the values of the next round
-			$bid02ForPersonI = 2 * $epsilon if ( $Opt01ValForPersonI > ($Opt02ValForPersonI + $epsilon) or $Opt02ValForPersonI > ($Opt03ValForPersonI + $epsilon) );
-
-			$info{$Opt02ObjForPersonI}{$bid02ForPersonI} = $person if ( ++$seen_bid{$Opt01ObjForPersonI}{$bid01ForPersonI} >= 2 );
-
+			if ( (not defined $info{$Opt01ObjForPersonI}) || ($bid01ForPersonI > $info{$Opt01ObjForPersonI}{'bid'}) ) 
+			{			
+				$info{$Opt01ObjForPersonI}{'bid'   } = $bid01ForPersonI; # Stores the bidding info for future use
+				$info{$Opt01ObjForPersonI}{'person'} = $person;
+				
+				#printf " object = %3s ; person = %3s ; Max bid = %12.5f \n", $Opt01ObjForPersonI, $info{$Opt01ObjForPersonI}{'person'}, $info{$Opt01ObjForPersonI}{'bid'}; sleep 1;
+			}
+			
 			if ( $epsilon_scaling >= 6 and $epsilon_scaling % 2 == 0 and not $seen_person{$person}++ ) # filter the old list
 			{					
 				for my $object ( @objects_with_greater_benefits ){ # frequently check the quality of objects in the old list
@@ -709,15 +709,14 @@ sub auctionRound {
 				printf $output "<> PersonI = %3s ; %3s objects desired by this person (old list) = (@old_list) ; objects whose current values are still updated = (@updated_price) : %2s >= 1 ? \n", $person, scalar @old_list, scalar @updated_price;
 				printf $output "<> PersonI = %3s ; %3s objects desired by this person (new list) = (@new_list) $msg; \@best_3_objects = (@best_3_objects) \n", $person, scalar @new_list if ( defined $Opt03ObjForPersonI );
 				printf $output "<> PersonI = %3s chose ObjectJ = %3s ; \$bid01ForPersonI %10.5f = \$Opt01ValForPersonI %.5f - \$Opt02ValForPersonI %.5f + \$epsilon %.5f \n", $person, $Opt01ObjForPersonI, $bid01ForPersonI, $Opt01ValForPersonI, $Opt02ValForPersonI, $epsilon;
-				printf $output "<> PersonI = %3s chose ObjectJ = %3s ; \$bid02ForPersonI %10.5f = \$Opt02ValForPersonI %.5f - \$Opt03ValForPersonI %.5f + \$epsilon %.5f \n", $person, $Opt02ObjForPersonI, $bid02ForPersonI, $Opt02ValForPersonI, $Opt03ValForPersonI, $epsilon if ( defined $bid02ForPersonI );
 			}
 		}
 	}
 	
-	foreach my $object ( keys %info ){                                 # for each object, choose only the first bid.
-	foreach my $bid    ( sort { $b <=> $a } keys %{$info{$object}} ){  # descending order!!! --> the first bid is the highest bid.
-	
-		my $person       = $info{$object}{$bid};		
+	foreach my $object ( keys %info ){
+		my $bid    = $info{$object}{'bid'   };
+		my $person = $info{$object}{'person'};
+		
 		my $other_person = $assignned_object{$object}; # Find the other person who has objectJ and make them unassigned
 	   	   	   
 		if ( defined $other_person ) {
@@ -743,9 +742,7 @@ sub auctionRound {
 		if ( $verbose >= 8 ){
 			printf $output " --> Assigning to personI = %3s the objectJ = %3s with highestBidForJ = %10.5f and update the price vector ; \$assignned_person{%3s} = %3s ; \$price_object{%3s} = %.5f \n", $person, $object, $bid, $person, $assignned_person{$person}, $object, $price_object{$object};	
 		}
-	   
-		last;  # next object, choose only the highest bid for each object
-	}}
+	}
 	
 	if ( $verbose >= 9 ){
 		print $output "\n Final: Matrix Size N x M: $min_size x $max_size ; Num Iterations: $iter_count_global ; epsilon_scaling = $epsilon_scaling ; iter_count_local = $iter_count_local ; epsilon: $epsilon \n";
@@ -890,7 +887,7 @@ __END__
  for my $i ( 0 .. $N - 1 ){
    my @weight_function;		   
    for my $j ( 0 .. $M - 1 ){
-	  my $weight = sprintf( "%.0f", rand($r) );
+      my $weight = sprintf( "%.0f", rand($r) );
       push @weight_function, $weight;
    }
    push @input_matrix, \@weight_function;
@@ -1013,7 +1010,7 @@ __END__
 =head1 AUTHOR
 
     Claudio Fernandes de Souza Rodrigues
-	April 16, 2018
+	April 19, 2018
 	Sao Paulo, Brasil
 	claudiofsr@yahoo.com
 	
